@@ -13,42 +13,46 @@ pub enum Error {
   NotIpv4,
 }
 
-pub fn daemon<'a>(
+pub fn daemon(
   name: String,
   port: u16,
   duration: u32,
 ) -> Receiver<Result<(u16, Ipv4Addr, SocketAddrV4), String>> {
-  let mut local_ip = Ipv4Addr::UNSPECIFIED;
-  let mut pre_gateway = SocketAddrV4::new(local_ip, 0);
-
-  let seconds = Duration::from_secs(duration.into());
-  let duration = duration + 9;
-  let mut ext_port = port;
   let (s, r) = channel();
 
   spawn(async move {
+    let mut local_ip = Ipv4Addr::UNSPECIFIED;
+    let mut pre_gateway = SocketAddrV4::new(local_ip, 0);
+    let duration = duration + 9;
+    let seconds = Duration::from_secs(duration.into());
+    let mut ext_port = port;
+    let mut retryed = 0;
     loop {
       match upnp(&name, port, ext_port, duration).await {
         Ok((gateway, ip)) => {
           if ip != local_ip || gateway != pre_gateway {
+            retryed = 0;
             local_ip = ip;
             pre_gateway = gateway;
             let _ = s.send(Ok((ext_port, ip, gateway)));
           }
         }
         Err(err) => {
+          local_ip = Ipv4Addr::UNSPECIFIED;
           let err = err.root_cause();
-          if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
-            if ext_port == 65535 {
-              ext_port = 1025;
-            } else {
-              ext_port += 1;
+          if retryed < 9 {
+            if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
+              retryed += 1;
+              if ext_port == 65535 {
+                ext_port = 1025;
+              } else {
+                ext_port += 1;
+              }
+              continue;
             }
-            continue;
-          } else {
-            local_ip = Ipv4Addr::UNSPECIFIED;
-            let _ = s.send(Err(err.to_string()));
           }
+          retryed = 0;
+          let _ = s.send(Err(err.to_string()));
         }
       }
       sleep(seconds).await;
